#include "prescan.h"

void db_connect(void);
void db_disconnect(void);
void db_setup(void);
void send_query(char *);
void db_load_config(void);

void db_connect(void)
{
  if(!mysql_init(&mysql))
  {
		fprintf(stderr, "Cannot initialize MySQL");
		exit(-1);
	}

  if(!mysql_real_connect(&mysql, config.db_server,config.db_user, config.db_pass, config.db_database, 0, NULL, 0))
   {
      fprintf(stderr, "CANT CONNECT: %d: %s \n",mysql_errno(&mysql), mysql_error(&mysql));
      exit(-1);
   }
 
  if(config.debug) printf("Database connection successful.\n");
}

void db_disconnect(void)
{
	mysql_close(&mysql);
}
 

void db_setup(void)
{
	
	db_connect();
	

    send_query("DROP TABLE `prescanic_ip`;");

    send_query("CREATE TABLE `prescanic_ip` (`source_ip` varchar(30) NOT NULL,`ip` varchar(30) NOT NULL,`hostname` varchar(512) NOT NULL,  `reverse_name` varchar(512) NOT NULL,`scan_time` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,`ping_reply` varchar(5) NOT NULL,`fingerprint` varchar(256) NOT NULL,`os_type` varchar(256) NOT NULL,`ftp_syst` varchar(24) NOT NULL,`ftp_anonymous` varchar(5) NOT NULL, `prescanic_version` varchar(24) NOT NULL) ENGINE=MyISAM DEFAULT CHARSET=latin1;");
 
	send_query("DROP TABLE `prescanic_data`;");

    send_query("CREATE TABLE `prescanic_data` (`source_ip` varchar(30) NOT NULL, `ip` varchar(30)  NOT NULL,`port` int  NOT NULL,`protocol` varchar(4)  NOT NULL,`banner` BLOB  NOT NULL) ENGINE = MyISAM;");

	db_disconnect();
	
	printf("\nDatabase setup successful.\n");
	exit(0);

}

void send_query(char *query_string)
{
   if(config.debug) printf("Sending MYSQL query: [%s]\n",query_string);
	 
   if (mysql_query(&mysql, query_string))
	 {
      fprintf(stderr, "ERROR SENDING QUERY: %s [%s]\n", mysql_error(&mysql),query_string);

   }
}

void db_load_config(void)
{
  FILE *db_config_file;
  char string[256];
  char *ptr;

  db_config_file = fopen(MYSQLCONFIG_FILE,"r");

  if(!db_config_file)
  {
    perror("Error opening database file");
    exit(-1);
  }

  memset(string,'\0',256);

  while(fgets(string,256,db_config_file) != NULL)
  {
    if(strncmp(string,"server=",7) == 0)
    {
      /* grab server */
      ptr = (char*)string + 7;
      strip_line_feed(ptr);

      if(config.debug)
        printf("DB CONFIG - SERVER: [%s]\n",ptr);


      config.db_server = (char*)Malloc(strlen(ptr) +1);
      strncpy(config.db_server,ptr,strlen(ptr));

    }
    else if(strncmp(string,"database=",9) == 0)
    {
      /* grab database */
      ptr = (char*)string + 9;
      strip_line_feed(ptr);
      if(config.debug)
        printf("DB CONFIG - DATABASE: [%s]\n",ptr);

      config.db_database = (char*)Malloc(strlen(ptr) +1);
      strncpy(config.db_database,ptr,strlen(ptr));

    }
    else if(strncmp(string,"user=",5) == 0)
    {
      /* grab user */
      ptr = (char*)string + 5;
      strip_line_feed(ptr);
      if(config.debug)
        printf("DB CONFIG - USER: [%s]\n",ptr);

      config.db_user = (char*)Malloc(strlen(ptr) +1);
      strncpy(config.db_user,ptr,strlen(ptr));

    }
    else if(strncmp(string,"pass=",5) == 0)
    {
      /* grab pass */
      ptr = (char*)string + 5;
      strip_line_feed(ptr);
      if(config.debug)
        printf("DB CONFIG - PASS: [%s]\n",ptr);

      config.db_pass = (char*)Malloc(strlen(ptr) +1);
      strncpy(config.db_pass,ptr,strlen(ptr));

    }
    
    memset(string,'\0',256);

  }

  if(config.db_server == NULL || config.db_database == NULL || config.db_user == NULL || config.db_pass == NULL)
  {
    perror("Error with database configuration file");
    exit(-1);
  }

}
