#include "prescan.h"

#undef DEBUG_INIT

void usage(void);
void init(int ,char **);
void set_defaults(void);
void check_args(void);
void printbanner(void);

void init(int argc,char **argv)
{
  int i;
  int option;

  /* PRINT ELITE BANNER CREATED BY KKR */
  printbanner();

  /* SET DEFAULT CONFIG BEFORE ARGUMENT PARSING */
  set_defaults();

  /* ARGUMENT PARSING */
  while ((option = getopt (argc, argv, "h:f:it:d:p:swvlc")) != EOF)
  {
    switch(option)
    {
      /* SINGLE HOST SCAN */
      case 'h':
        config.single_host = (char*) Malloc(strlen(optarg)+1);
        sprintf(config.single_host,"%s\0",optarg);
        break;
      /* USE HOST FILE */
      case 'f':
        config.host_file = (char*) Malloc(strlen(optarg) + 1);
        sprintf(config.host_file,"%s\0",optarg);
        break;
      /* USE STDIN */
      case 'i':
        config.stdin_i = 1;
        break;
      /* HOST TIMEOUT LENGTH */
      case 't':
        config.timeout = atoi(optarg);
        break;
      /* DEVICE */
      case 'd':
        Free(config.device);
        config.device = (char*) Malloc(strlen(optarg) + 1);
        sprintf(config.device,"%s\0",optarg);
        break;
      /* # OF FORK PROCESSES */
      case 'p':
        config.fork_num = atoi(optarg);
        break;
      /* STEALTH MODE */
      case 's':
        config.stealth = 1;
        break;
      case 'w':
      /* PRINT OUTPUT TO STDOUT */
        config.stdout_o = 1;
        break;
      /* VERBOSE (DEBUG) */
      case 'v':
        config.debug = 1;
        break;
      /* DON'T REVSERSE LOOKUP */
      case 'l':
        config.r_lookup = 1;
        break;
	  /* SETUP DATABASE ONLY */
      case 'c':
      	config.db_setup = 1;
      	break;
        
    }
  }

  /* LOAD DATABASE CONFIG FILE */
  db_load_config();

  /* DATABASE SETUP, exit() is called from this subroutine */
  if(config.db_setup) db_setup();
  
  /* CHECK FOR ROOT */
  if(geteuid() == 0) config.got_root = 1;
  else config.got_root = 0;
  
  /* CHECK FOR CONFLICTING ARGUMENTS */
  check_args();

  /* INITIATE SNIFFER */
  if(config.got_root)
    init_pcap();
    
  /* GET LOCAL IP OF SELECTED DEVICE */
  if(config.got_root)
    config.local_ip = get_device_ip(config.device);

  /* PRINT DESTINATION PORTS */
  printf("*** Ports:");
  for(i = 0; config.port[i] != 0 ; ++i) printf(" [%d]",config.port[i]);
  printf("\n");
}


void usage(void)
{
  printf("\nUSAGE:\n");
  printf("# prescanic <[-h host][-f host_file][-i]> [options]\n");
  printf(" -h <host>\tScan a single host.\n");
  printf(" -f <host_file>\tAccept hosts from \"host_file\"\n");
  printf(" -i \t\tAccept host's from STDIN.\n");
  printf("OPTIONS:\n");
  printf(" -s \t\tStealth mode (syn scan, os detection, nothing else).\n");
  printf(" -t <timeout>\t# of seconds to wait after receiving no data from the host.\n");
  printf(" -d <device>\tNetwork device used to sniff and send on.\n");
  printf(" -p <processes>\tNumber of processes to fork at one time. (0 means don't fork)\n");
  printf(" -v \t\tVerbose (lots of debug information)\n");
  printf(" -w \t\tWrite scan to STDOUT (as well as save it).\n");
  printf(" -l \t\tDon't reverse lookup IP addresses.\n");
  printf("DATABASE CONFIG:\n");
  printf(" -c \t\tDon't do anything but create tables for mySQL database.\n");
  printf("    \t\t(WARNING: if the tables have already been created, they will be dropped and recreated.\n");
  printf("\n");
  exit(-1);
}


void set_defaults(void)
{
  /* HARDCODED CONFIGURATION SETTINGS  :) */

  bzero(&config,sizeof(config));

  config.device = (char*) Malloc(8);
  sprintf(config.device, "default\0");

  config.timeout = 7;  /* in seconds... */
  config.savescan = 1;    /* stdout and an output file! */

  config.port[0] = 21;
  config.port[1] = 22;
  config.port[2] = 23;
  config.port[3] = 25;
  config.port[4] = 53;
  config.port[5] = 80;
  config.port[6] = 110;
  config.port[7] = 111;
  config.port[8] = 139;
  config.port[9] = 143;
  config.port[10] = 515;
  config.port[11] = 443;
  config.port[12] = 6000;

}

void check_args(void)
{
  int i=0;
  
  i = config.stdin_i;
  if(config.host_file) ++i;
  if(config.single_host) ++i;

  if(i>1)
  {
    printf("You must choose between -h,-f, or -i!\n");
    usage();
  }

  if(i==0)
  {
    printf("Prescanic found no tasks.  Please read program usage.\n");
    usage();
  }

}


void printbanner(void)
{
  FILE *bannerfd;
  char thebanner[512];
  
  bannerfd = fopen("banner","r");

  while(fgets(thebanner,sizeof(thebanner),bannerfd))
    printf("%s",thebanner);

  fclose(bannerfd);
}
  
