#include "prescan.h"

int main(int,char **);


/* GLOBAL VARIABLES */
struct config config;
int datalink;
pcap_t *pd;
MYSQL mysql;


int main(int argc, char **argv)
{
  int fork_retval,fork_counter=1,prescan_retval;
  FILE *hostfile;
  char string[256];
  struct host *victim;

  /* BANNER, LOAD CONFIG, PARSE ARGUMENTS, SET PORTS, ETC */
  init(argc,argv);

  /* SINGLE HOST SCAN */
  if(config.single_host)
  {
    victim = (struct host*) add_victim(config.single_host);
    prescan_retval = prescan(victim);
    if(prescan_retval < 0) exit(EXIT_FAILURE);
    if(config.got_root)
      determine_os(victim);      /* determine OS from fingerprint responses */
    output(victim);
    del_victim(victim);
    exit(EXIT_SUCCESS);
  }

  else if(config.host_file || config.stdin_i)
  {
    /* SCAN USING HOST FILE */
    if(config.host_file)
    {
      hostfile = fopen(config.host_file,"r");
      if(!hostfile)
      {
        perror("Error reading specified host file");
        exit(EXIT_FAILURE);
      }
    }
    /* SCAN USING STDIN */
    else
      hostfile = stdin;


    while(fgets(string,255,hostfile) != NULL)
    {
      if(!isalnum(string[0])) continue;    /* first char should be alphanumeric */

      string[strlen(string)-1] = 0;   /* strip \n */

      /* DONT FORK */
      if(config.fork_num == 0)
      {
        victim = (struct host*) add_victim(string);
        prescan_retval = prescan(victim);
        if(prescan_retval < 0) continue;
        if(config.got_root)
          determine_os(victim);      /* determine OS from fingerprint responses */
        output(victim);
        del_victim(victim);
        continue;
      }


      /* FORK */
      else if(config.fork_num > 0)
      {
        fork_retval = fork();

        if(fork_retval == -1)
        {
          perror("fork() failed");
          exit(EXIT_FAILURE);
        }
        /* CHILD */
        else if(fork_retval == 0)
        {
          fclose(hostfile);   /* THIS LINE OWNZ ME */
          victim = (struct host*) add_victim(string);
          prescan(victim);
          if(config.got_root)
            determine_os(victim);      /* determine OS from fingerprint responses */
          output(victim);
          exit(EXIT_SUCCESS);
        }
        /* PARENT */
        else if(fork_retval > 0)
        {
          if(config.debug)
            printf("FORK:>>>>> [CHILD SPAWNED] (pid: %d)\n",fork_retval);

          if(fork_counter >= config.fork_num)
          {
            if(config.debug)
              printf("FORK:>>>>> wait()'ing. (%d)\n",getpid());

            wait(NULL);
            usleep(1000);
          }
          else
            ++fork_counter; /* putting 'fork_counter' in the if statement should prevent
                              a bug when scanning from HUGE host files. */
        }
      }
    }
    printf("Multiple host scan from file [%s] completed. Exiting.\n",config.host_file);
    fclose(hostfile);
    exit(EXIT_SUCCESS);
  }


  exit(EXIT_FAILURE);

}
