#include "prescan.h"

void forward_lookup(struct host *);
void reverse_lookup(struct host *);
int nonblock_connect(u_long ,int );
int sock_status(int );
int readline_from_sock(int ,char *,int);
u_long get_device_ip (char *);
char *Inet_ntoa(u_long);
int send_raw_packet(struct iphdr *,struct tcphdr *,struct udphdr *,struct icmphdr *,char *,int,int,int);
unsigned short in_cksum(unsigned short *,int);

int send_raw_packet(struct iphdr *ip_hdr, struct tcphdr *tcp_hdr,
                          struct udphdr *udp_hdr, struct icmphdr *icmp_hdr,
                              char *data, int datalen, int sendto_flags, int set_checksums)
    {

        int rawsock,packetsize=0;
        int on=1;
        char *ptr,*packet;
        struct sockaddr_in sin;

        /* NEEDED FOR CHECKSUM CALCULATIONS... */
        struct iphdr *ip_tmp;
        struct tcphdr *tcp_tmp;
        struct udphdr *udp_tmp;
        struct icmphdr *icmp_tmp;
        struct pseudohdr *pseudo_header;


        if(!ip_hdr) return -1;
        if(tcp_hdr && udp_hdr) return -1;
        if(tcp_hdr && icmp_hdr) return -1;
        if(udp_hdr && icmp_hdr) return -1;
        if(datalen && !data) datalen = 0;

        packetsize += 20;	/* STANDARD IP HEADER LENGTH */
        if(tcp_hdr) packetsize += 20;	/* STANDARD TCP HEADER LENGTH */
        else if(udp_hdr) packetsize += 8;	/* STANDARD UDP HEADER */
        else if(icmp_hdr) packetsize += 8;	/* STANDARD ICMP HEADER */

        if(data && datalen > 0) packetsize += datalen;

        packet = (char *) Malloc(packetsize + 1);	/* 3 padding.. */
        bzero(packet,packetsize+1);

        ptr = packet;

        bcopy(ip_hdr,ptr,20); ptr += 20;
        if(tcp_hdr) { bcopy(tcp_hdr,ptr,20); ptr += 20; }
        else if(udp_hdr) { bcopy(udp_hdr,ptr,8); ptr += 8; }
        else if(icmp_hdr) { bcopy(icmp_hdr,ptr,8); ptr += 8; }
        if(data && datalen > 0) { bcopy(data,ptr,datalen); ptr += datalen; }

        if(set_checksums)
        {
          ip_tmp = (struct iphdr*)packet;

          if(tcp_hdr)
          {
            tcp_tmp = (struct tcphdr*)(packet + sizeof(struct iphdr));

            /* this is gonna break the iphdr, so we fix it later. */
            pseudo_header = (struct pseudohdr*)(packet + 8);  /* 20 - 12 */

            /* use ip_hdr, because ip_tmp is currently broken! */
            pseudo_header->saddr = ip_hdr->saddr;
            pseudo_header->daddr = ip_hdr->daddr;
            pseudo_header->mbz = 0;
            pseudo_header->protocol = ip_hdr->protocol;
            pseudo_header->tcp_length = htons(sizeof(struct iphdr));

            tcp_tmp->check = in_cksum((u_short *)pseudo_header,33+datalen); /* 33=tcp+pseudo+1 byte buffer */

            /* fix the ip header */
            memcpy( ip_tmp,ip_hdr,sizeof(struct iphdr));
          }
          else if(udp_hdr)
          {
            /* CALCULATE UDP HEADER */
          }
          else if(icmp_hdr)
          {
            icmp_tmp = (struct icmphdr*)(packet + sizeof(struct iphdr));
            icmp_tmp->checksum = in_cksum ((u_short*)icmp_tmp,9+datalen);   /* 9=icmp+1 byte buffer */
          }

          /* ip header checksums calculated *later*, incase os pseudo header shit*/
          ip_tmp->check = in_cksum((unsigned short *)ip_tmp,20);

        }   /* end of if(set_checksums) */

        sin.sin_family = AF_INET;
        sin.sin_addr.s_addr = ip_hdr->daddr;
        if(tcp_hdr) sin.sin_port = tcp_hdr->dest;

        /* CREATE RAW SOCKET. */
        if((rawsock = socket(AF_INET, SOCK_RAW, IPPROTO_RAW)) == -1)
          return -1;

        /* SET SOCKET OPTIONS. */
        if (setsockopt(rawsock, IPPROTO_IP, IP_HDRINCL, &on, sizeof(on)) < 0)
          return -1;

        /* SEND PACKET */
        if(sendto(rawsock, packet, packetsize, sendto_flags, &sin, sizeof(sin)) < 0)
          return -1;

        free(packet);
        close(rawsock);

        return 1;

    }

u_short in_cksum(unsigned short *ptr,int nbytes)
{

  register long     sum;            /* assumes long == 32 bits */
  u_short                 oddbyte;
  register u_short        answer;         /* assumes u_short == 16 bits */

  /*
   * Our algorithm is simple, using a 32-bit accumulator (sum),
   * we add sequential 16-bit words to it, and at the end, fold back
   * all the carry bits from the top 16 bits into the lower 16 bits.
   */

  sum = 0;
  while (nbytes > 1)  {
    sum += *ptr++;
    nbytes -= 2;
  }

  /* mop up an odd byte, if necessary */
  if (nbytes == 1) {
    oddbyte = 0;            /* make sure top half is zero */
    *((u_char *) &oddbyte) = *(u_char *)ptr;   /* one byte only */
    sum += oddbyte;
  }

  /*
   * Add back carry outs from top 16 bits to low 16 bits.
   */

  sum  = (sum >> 16) + (sum & 0xffff);    /* add high-16 to low-16 */
  sum += (sum >> 16);                     /* add carry */
  answer = ~sum;          /* ones-complement, then truncate to 16 bits */
  return(answer);
}


void forward_lookup(struct host *victim)
{
  struct hostent *host;
  struct in_addr inp;

  if(inet_aton(victim->host,&inp))
  {
    victim->ip = inp.s_addr;
    return;
  }

  if((host = gethostbyname(victim->host)) == NULL)
    return;

  memcpy(&victim->ip,host->h_addr,host->h_length);

  return;
}

void reverse_lookup(struct host *victim)
{
  struct hostent *host;
  struct in_addr addr;

  if(!victim->ip) return;

  addr.s_addr = victim->ip;

  if((host = gethostbyaddr ((char*) &addr, sizeof(struct in_addr), AF_INET)) == NULL)
  {
    victim->name = victim->host;
    return;
  }

  victim->name = (char*) Malloc(strlen(host->h_name) +1);
  bzero(victim->name,(strlen(host->h_name) + 1));
  sprintf(victim->name,"%s\0",host->h_name);

  return;
}


int nonblock_tcp_connect(u_long ip,int port)
{
  int sockfd;
  int sock_flags;

  struct sockaddr_in addr;

  addr.sin_port = htons(port);	/* convert to nbo */
  addr.sin_family = AF_INET;
  addr.sin_addr.s_addr = ip;

  if((sockfd = socket(AF_INET,SOCK_STREAM,0)) < 0)
  {
    perror("socket() failed");
    exit(EXIT_FAILURE);
  }

  /* SET TO NON BLOCKING... */
  sock_flags = fcntl(sockfd,F_GETFL);
  if(sock_flags >= 0) 
  {
      if(fcntl(sockfd,F_SETFL,sock_flags | O_NONBLOCK) < 0)
      {
        perror("fcntl1() failed");
        exit(EXIT_FAILURE);   
      }
  }
  else
  {
      perror("fcntl2() failed");
      exit(EXIT_FAILURE);
  }

  if(connect(sockfd,(struct sockaddr *) &addr, sizeof(addr)) < 0)
  {
    if(errno == EINPROGRESS)
      return sockfd;	/* its non blocking socket */
    else
    {
      perror("connect() failed");
      exit(EXIT_FAILURE);
    }
  }

  return sockfd;

}


int nonblock_udp_connect(u_long ip,int port)
{
  int sockfd;
  int sock_flags;

  struct sockaddr_in addr;

  addr.sin_port = htons(port);	/* convert to nbo */
  addr.sin_family = AF_INET;
  addr.sin_addr.s_addr = ip;

  if((sockfd = socket(AF_INET,SOCK_DGRAM,0)) < 0)
  {
    perror("socket() failed");
    exit(EXIT_FAILURE);
  }

  /* SET TO NON BLOCKING... */
  sock_flags = fcntl(sockfd,F_GETFL);
  if(sock_flags >= 0) 
  {
      if(fcntl(sockfd,F_SETFL,sock_flags | O_NONBLOCK) < 0)
      {
        perror("fcntl3() failed");
        exit(EXIT_FAILURE);   
      }
  }
  else
  {
      perror("fcntl4() failed");
      exit(EXIT_FAILURE);
  }


  if(connect(sockfd,(struct sockaddr *) &addr, sizeof(addr)) < 0)
  {
    if(errno == EINPROGRESS)
      return sockfd;	/* its non blocking socket */
    else
    {
      perror("connect() failed");
      exit(EXIT_FAILURE);
    }
  }

  return sockfd;

}

int sock_status(int sockfd)
{
  int sockopt = -1,datalen=0,socklen=4;
  fd_set wset;
  struct timeval tv;
  static char blah[512];

  tv.tv_sec = 0;
  tv.tv_usec = 0;

  FD_ZERO (&wset);

  FD_SET (sockfd, &wset);

  select ((sockfd + 1), NULL, &wset, NULL, &tv);

  if (FD_ISSET (sockfd, &wset))
    getsockopt (sockfd, SOL_SOCKET, SO_ERROR, &sockopt, &socklen);

  if (sockopt == -1)      /* Operation now in progress, no status! */
    return PROGRESS;
  else if (sockopt == 0)
  {
    datalen = recv(sockfd,blah,512,MSG_PEEK);
    if(datalen > 0) return datalen;
    else return CONNECTED;			/* connection successful */
  }
  else if (sockopt == 111)
    return REJECTED;           /* connection rejected */
  else if (sockopt == 113)
    return NOROUTE;           /* no route to host */
  else if (sockopt == 101)
    return N_UNREACH;           /* network unreachable */
  else if (sockopt == 104)
    return PEER_RESET;          /* connection reset by peer */

  else
  {
    if(config.debug)
      printf("sock_status() is returning %d on errno %d\n",UNKNOWN,sockopt);
    return UNKNOWN;           /* dunno what happened, try strerror(sockopt) ! */
  }
}

int readline_from_sock(int sock,char *line,int max_read)
{
  int 	i=0,retval=0;

  bzero(line,max_read);
  retval = recv(sock,line,max_read,MSG_PEEK);
  if(retval < 0) return -1;
  while(line[i] != '\n' && i != max_read && i != retval) ++i;
  retval = read(sock,line,++i);
  line[i] = 0;    /* terminate the string */

  return retval;

}

u_long get_device_ip (char *device)
{

  int sock, count, traverse;
  struct sockaddr_in * so_addr;
  char buf[512];
  struct ifreq *i;
  struct ifconf ic;

  sock = socket (AF_INET, SOCK_STREAM, 0);

  ic.ifc_len = 512;
  ic.ifc_buf = buf;
  ioctl (sock, SIOCGIFCONF, (char *) &ic);
  i = ic.ifc_req;
  count = ic.ifc_len / sizeof (struct ifreq);
  for (traverse = 0; traverse < count; traverse++)
  {
    ioctl (sock, SIOCGIFADDR, (char *) &i);
    so_addr = (struct sockaddr_in *) &i->ifr_ifru.ifru_addr;
    if (!strcmp (i->ifr_name, device))
      return so_addr->sin_addr.s_addr;
    i++;
  }
  return 0;
}

char *Inet_ntoa(u_long ip)
{
  struct in_addr tmp_addr;
  tmp_addr.s_addr = ip;
  return inet_ntoa(tmp_addr);
}
