#include "prescan.h"

void init_osdetect(struct host *);
void handle_response(struct host *,int ,struct tcphdr *);
void determine_os(struct host *);

void init_osdetect(struct host *victim)
{
  /* send "special packets"... */

  if(config.debug)
    printf("init_osdetect() running. (LOCAL IP: %s)\n",Inet_ntoa(config.local_ip));

  /* BEGIN SENDING PACKETS */

  send_tcp_stack_fingerprint(victim,PORT_1,1,1,0,0);
  send_tcp_stack_fingerprint(victim,PORT_2,0,0,1,0);
  send_tcp_stack_fingerprint(victim,PORT_3,0,1,1,0);
  send_tcp_stack_fingerprint(victim,PORT_4,1,0,1,0);
  send_tcp_stack_fingerprint(victim,PORT_5,0,0,0,1);
}

void handle_response(struct host *victim,int i,struct tcphdr *tcp)
{
  if(victim->response[i] != 0) return;  /* we've already handled a response from this port */

  if(tcp->syn)        victim->response[i] += SYN;
  if(tcp->ack)        victim->response[i] += ACK;
  if(tcp->fin)        victim->response[i] += FIN;
  if(tcp->psh)        victim->response[i] += PSH;
  if(tcp->urg)        victim->response[i] += URG;
  if(tcp->rst)        victim->response[i] += RST;
  if(tcp->window)     victim->response[i] += WINDOW;
  if(tcp->seq)        victim->response[i] += SEQNUM;
  if(tcp->ack_seq)    victim->response[i] += ACKNUM;

  if(tcp->window) victim->response_window[i] = ntohs(tcp->window);

  if(config.debug)
    printf("FINGERPRINT RESPONSE: packet#(%d) port#(%d) fingerprint#(%d) window#(%d)\n",(i+1),ntohs(tcp->dest),victim->response[i],ntohs(tcp->window));
}

void determine_os(struct host *victim)
{
  /* fingerprint file is opened everytime so you can update it on the fly,
     plus we save memory by not sticking the entire fingerprint list into
     some kind of dynamic structure. */

  FILE *fingerprints;
  char finger[128];
  char *ptr;

  fingerprints = fopen(FINGERPRINTS_FILE,"r");
  if(!fingerprints)
  {
    perror("Error opening fingerprint file");
    exit(-1);
  }

  bzero(victim->fingerprint_string,sizeof(victim->fingerprint_string));
  sprintf(victim->fingerprint_string,"%d:%d:%d:%d:%d:%d:%d:%d:%d:%d\0",victim->response[0],
           victim->response[1],victim->response[2],victim->response[3],
           victim->response[4],victim->response_window[0],victim->response_window[1],
           victim->response_window[2],victim->response_window[3],victim->response_window[4]);

  if(config.debug)
    printf("VICTIM'S FINGERPRINT: %s\n", victim->fingerprint_string);

  while(fgets(finger,128,fingerprints) != NULL)
  {
    if(!isdigit(finger[0])) continue;   /* first char should be a digit */
    /* FIND RESPONSE #1 */
    ptr = strtok(finger,":");
    if(atoi(ptr) != victim->response[0]) continue;
    /* FIND RESPONSE #2 */
    ptr += (strlen(ptr) + 1);
    ptr = strtok(ptr,":");
    if(atoi(ptr) != victim->response[1]) continue;
    /* FIND RESPONSE #3 */
    ptr += (strlen(ptr) + 1);
    ptr = strtok(ptr,":");
    if(atoi(ptr) != victim->response[2]) continue;
    /* FIND RESPONSE #4 */
    ptr += (strlen(ptr) + 1);
    ptr = strtok(ptr,":");
    if(atoi(ptr) != victim->response[3]) continue;
    /* FIND RESPONSE #5 */
    ptr += (strlen(ptr) + 1);
    ptr = strtok(ptr,":");
    if(atoi(ptr) != victim->response[4]) continue;


    /* FIND RESPONSE_WINDOW #1 */
    ptr += (strlen(ptr) + 1);
    ptr = strtok(ptr,":");
    if(atoi(ptr) != victim->response_window[0]) continue;
    /* FIND RESPONSE_WINDOW #2 */
    ptr += (strlen(ptr) + 1);
    ptr = strtok(ptr,":");
    if(atoi(ptr) != victim->response_window[1]) continue;
    /* FIND RESPONSE_WINDOW #3 */
    ptr += (strlen(ptr) + 1);
    ptr = strtok(ptr,":");
    if(atoi(ptr) != victim->response_window[2]) continue;
    /* FIND RESPONSE_WINDOW #4 */
    ptr += (strlen(ptr) + 1);
    ptr = strtok(ptr,":");
    if(atoi(ptr) != victim->response_window[3]) continue;
    /* FIND RESPONSE_WINDOW #5 */
    ptr += (strlen(ptr) + 1);
    ptr = strtok(ptr,":");
    if(atoi(ptr) != victim->response_window[4]) continue;


    /* IF WE'VE MADE IT THIS FAR, THE OS TYPE HAS BEEN FOUND! */
    ptr += (strlen(ptr) + 1);
    victim->os = (char *) Malloc(strlen(ptr));
    if(config.debug) printf("Operating system found: %s\n",ptr);
    snprintf(victim->os,strlen(ptr),"%s\0",ptr);
    break;
  }
  fclose(fingerprints);

}
