#include "prescan.h"

void init_pcap(void);
void die_pcap(char *);
char *next_packet(void);

struct pcap_pkthdr hdr;

void init_pcap(void)
{
  unsigned int localnet,netmask;
#define INET_ADDRSTRLEN	16	/* ddd.ddd.ddd.ddd\0 */
  char errbuffer[PCAP_ERRBUF_SIZE];
  struct bpf_program fcode;

  if(strncmp(config.device,"default",6) == 0)
  {
    config.device = pcap_lookupdev(errbuffer);
    if(!config.device) die_pcap(errbuffer);
  }

  if(config.debug)
    printf("[PCAP] Device is [%s]\n",config.device);

  if((pd = pcap_open_live(config.device,SNAPLEN,1,500,errbuffer)) == NULL)
    die_pcap(errbuffer);

  if(pcap_lookupnet(config.device,&localnet,&netmask,errbuffer) < 0)
    die_pcap(errbuffer);

  if(pcap_compile(pd, &fcode,CMD,0,netmask) < 0)
    die_pcap(pcap_geterr(pd));
  if(pcap_setfilter(pd, &fcode) < 0)
    die_pcap(pcap_geterr(pd));

  if((datalink = pcap_datalink(pd)) < 0)
    die_pcap(pcap_geterr(pd));

  if(config.debug)
    printf("[PCAP] Datalink number is %d\n",datalink);

}

void die_pcap(char *errbuf)
{
  if(errbuf) perror(errbuf);
  exit(-1);
}

char *next_packet(void)
{

  char *ptr;
  int retval=0;
  fd_set packets;
  struct timeval tv;

  tv.tv_sec = 0;
  tv.tv_usec = 50;

  FD_ZERO(&packets);
  FD_SET(pcap_fileno(pd),&packets);
  retval = select((pcap_fileno(pd)+1),&packets,NULL,NULL,&tv);
  if(!retval) return NULL;

  /* pcap_next() seems to be blocking on ppp0, and nonblocking on eth0.
      I have appended an if(!ptr) statement incase we're using something
      similar to eth0. */

  ptr = (char*) pcap_next(pd,&hdr);
  if(!ptr) return NULL;

  switch(datalink)
  {
    case DLT_RAW:
      return ptr;
      break;

    case DLT_EN10MB:
      return (ptr + 14);
      break;

    case DLT_PPP:
      return (ptr + 9);
      break;

      default:
      printf("Unsupported datalink, sorry!\n");
      exit(-1);
  }

}
