#include "prescan.h"

int tcp_check(struct host *);
void add_tcp(int,int,struct host *);
void del_tcp(struct tcp_connection *);
char *status_string(struct tcp_connection *);
void tcp_close(struct tcp_connection *);

int tcp_check(struct host *victim)
{
  int retval=0;

  for(victim->tcp_current = victim->tcp_first; victim->tcp_current ; victim->tcp_current = victim->tcp_current->next)
  {
    /* CONTINUE: if we've already been rejected, dont check it. */
    if(victim->tcp_current->state < CONNECTED && victim->tcp_current->state != PROGRESS) continue;

    victim->tcp_current->state = sock_status(victim->tcp_current->sock);

    if(victim->tcp_current->state >= CONNECTED && !victim->tcp_current->tool)
    {
      if(config.debug)
        printf("[%d] CONNECTION ACCEPTED\n",victim->tcp_current->port);
        
      victim->tcp_current->open = 1;
      victim->tcp_current->tool = 1;
      talk_with_protocol(victim);
      retval = 1; /* reset the timeout */
      
      /* INIT: OS DETECTION */
      if(config.got_root && !victim->osdetect_tool)
      {
        init_osdetect(victim);
        victim->osdetect_tool = 1;
      }
    }

    if(victim->tcp_current->state > CONNECTED)
    {
      /* DATA WAS FOUND ON SOCK */
      if(victim->tcp_current->datalen != victim->tcp_current->state)
      {
        /* *NEW* DATA RECEIVED */
        if(config.debug)
          printf("[%d] NEW DATA FOUND (%d)\n",victim->tcp_current->port,victim->tcp_current->state);

        victim->tcp_current->datalen = victim->tcp_current->state;
        talk_with_protocol(victim);
        retval = 1; /* reset the timeout */
      }
      victim->tcp_current->datalen = victim->tcp_current->state;
    }

    if(config.debug > 0)
    {
      if(victim->tcp_current->state == LOCAL_RESET)
        printf("[%d] Connection closed localy.\n",victim->tcp_current->port);
      else if(victim->tcp_current->state < PROGRESS)
        printf("[%d] Error connecting: %s\n",victim->tcp_current->port,status_string(victim->tcp_current));
    }
  }   /* END OF FOR(TCP_CURRENT->NEXT) LOOP */

  return retval;
}

void add_tcp(int src_port,int port,struct host *victim)
{

  victim->tcp_current = (struct tcp_connection *) Malloc(sizeof(struct tcp_connection));

  memset(victim->tcp_current,0,sizeof(struct tcp_connection));
  
  victim->tcp_current->next = NULL;
  victim->tcp_current->previous = NULL;

  if(!victim->tcp_first) victim->tcp_first = victim->tcp_current;
  
  victim->tcp_current->port = port;
  
  if(config.stealth)
    send_syn(src_port,port,victim);
  else
    victim->tcp_current->sock = nonblock_tcp_connect(victim->ip,victim->tcp_current->port);
  
  victim->tcp_current->state = PROGRESS;    /* not connected, in progress */
  if(victim->tcp_previous)
  {
    victim->tcp_previous->next = victim->tcp_current;
    victim->tcp_current->previous = victim->tcp_previous;
  }

  victim->tcp_previous = victim->tcp_current;

}

void del_tcp(struct tcp_connection *tcp_current)
{

  tcp_current->previous->next  = tcp_current->next;
  tcp_current->next->previous = tcp_current->previous;
  close(tcp_current->sock);
  Free(tcp_current);

}

void tcp_close(struct tcp_connection *tcp_current)
{
  close(tcp_current->sock);
  tcp_current->state = LOCAL_RESET;
}
char *status_string(struct tcp_connection *tcp_current)
{
  static char status[64];

  if(tcp_current->state > PEER_RESET)
    sprintf(status,"OPEN\0");
  else if(tcp_current->state == PROGRESS)
    sprintf(status,"CLOSED (no response, possibly filtered)\0");
  else if(tcp_current->state == REJECTED)
    sprintf(status,"CLOSED (rejected)\0");
  else if(tcp_current->state == NOROUTE)
    sprintf(status,"CLOSED (no route to host)\0");
  else if(tcp_current->state == N_UNREACH)
    sprintf(status,"CLOSED (network unreachable)\0");
  else if(tcp_current->state == PEER_RESET)
    sprintf(status,"OPEN   (reset by peer, possibly tcpwrappers)\0");
  else if(tcp_current->state == UNKNOWN)
    sprintf(status,"CLOSED (?)\0");
  else
    sprintf(status,"? (STATE: %d)\0",tcp_current->state);
  return status;
}
 
